from .timeserieswidget import TimeseriesWidget, precomputeMultiscaleRecordings
from .templatewidget import TemplateWidget
from .featurespacewidget import FeatureSpaceWidget, FeatureSpaceWidgetPlotly
from .electrodegeometrywidget import ElectrodeGeometryWidget
from .unitwaveformswidget import UnitWaveformWidget, UnitWaveformsWidget
from .correlogramswidget import CorrelogramsWidget
