from .widgets.timeserieswidget import TimeseriesWidget
from .widgets.featurespacewidget import FeatureSpaceWidget
from .widgets.unitwaveformswidget import UnitWaveformsWidget
from .widgets.sortingaccuracywidget import SortingAccuracyWidget
from .widgets.crosscorrelogramswidget import CrossCorrelogramsWidget
from .widgets.electrodegeometrywidget import ElectrodeGeometryWidget

from .tables.sortingcomparisontable import SortingComparisonTable

from .validation.sortingcomparison import SortingComparison

from . import example_datasets
from . import lazyfilters
from . import devel
